__title__ = 'revolution'
__version__ = '0.1.0'
__author__ = 'Giovanni Salinas'
__author_email__ = 'gbs3@protonmail.com'
__description__ = 'An assortment of spinners to use while your Python programs run.'
__url__ = 'https://github.com/GBS3/revolution'
__license__ = 'MIT License'
__copyright__ = 'Copyright 2021 Giovanni Salinas'
